@@ -1,5 +1,5 @@
 package Beam::Wire;
-$Beam::Wire::VERSION = '1.017';
+$Beam::Wire::VERSION = '1.018';
 # ABSTRACT: Lightweight Dependency Injection Container
 
 
@@ -14,6 +14,8 @@ use Data::DPath qw ( dpath );
 use Path::Tiny qw( path );
 use File::Basename qw( dirname );
 use Types::Standard qw( :all );
+use Data::Dumper;
+use constant DEBUG => $ENV{BEAM_WIRE_DEBUG};
 
 
 has file => (
@@ -79,6 +81,9 @@ has meta_prefix => (
 
 sub get {
     my ( $self, $name, %override ) = @_;
+
+    ; print STDERR "Get service: $name\n" if DEBUG;
+
     if ( $name =~ q{/} ) {
         my ( $container_name, $service ) = split m{/}, $name, 2;
         return $self->get( $container_name )->get( $service, %override );
@@ -90,6 +95,8 @@ sub get {
 
     my $service = $self->services->{$name};
     if ( !$service ) {
+        ; printf STDERR 'Service "%s" does not exist. Creating.' . "\n", $name if DEBUG;
+
         my $config_ref = $self->get_config($name);
         unless ( $config_ref ) {
             Beam::Wire::Exception::NotFound->throw(
@@ -98,7 +105,9 @@ sub get {
             );
         }
 
-        if ( ref $config_ref eq 'HASH' && $self->is_meta( $config_ref ) ) {
+        ; print STDERR "Got service config: " . Dumper $config_ref if DEBUG;
+
+        if ( ref $config_ref eq 'HASH' && $self->is_meta( $config_ref, 1 ) ) {
             my %config  = %{ $self->normalize_config( $config_ref ) };
             $service = $self->create_service( $name, %config );
             if ( !$config{lifecycle} || lc $config{lifecycle} ne 'factory' ) {
@@ -110,6 +119,8 @@ sub get {
         }
     }
 
+    ; print STDERR "Returning service: " . Dumper $service if DEBUG;
+
     return $service;
 }
 
@@ -143,6 +154,8 @@ sub get_config {
 sub normalize_config {
     my ( $self, $conf ) = @_;
 
+    ; print STDERR "In conf: " . Dumper $conf if DEBUG;
+
     my %meta = reverse $self->get_meta_names;
 
     # Confs without prefixed keys can be used as-is
@@ -158,6 +171,8 @@ sub normalize_config {
         }
     }
 
+    ; print STDERR "Out conf: " . Dumper \%out_conf if DEBUG;
+
     return \%out_conf;
 }
 
@@ -165,6 +180,8 @@ sub normalize_config {
 sub create_service {
     my ( $self, $name, %service_info ) = @_;
 
+    ; print STDERR "Creating service: " . Dumper \%service_info if DEBUG;
+
     # Compose the parent ref into the copy, in case the parent changes
     %service_info = $self->merge_config( %service_info );
 
@@ -192,6 +209,14 @@ sub create_service {
         return $self->_load_config( "$conf_path" );
     }
 
+    if ( !$service_info{class} ) {
+        Beam::Wire::Exception::InvalidConfig->throw(
+            name => $name,
+            file => $self->file,
+            error => 'Service configuration incomplete. Missing one of "class", "value", "config"',
+        );
+    }
+
     use_module( $service_info{class} );
 
     if ( my $with = $service_info{with} ) {
@@ -326,6 +351,8 @@ sub parse_args {
 sub find_refs {
     my ( $self, $for, @args ) = @_;
 
+    ; printf STDERR qq{Searching for refs for "%s": %s}, $for, Dumper \@args if DEBUG;
+
     my @out;
     my %meta = $self->get_meta_names;
     for my $arg ( @args ) {
@@ -335,6 +362,8 @@ sub find_refs {
                     push @out, $self->resolve_ref( $for, $arg );
                 }
                 else { # Try to treat it as a service to create
+                    ; print STDERR "Creating anonymous service: " . Dumper $arg if DEBUG;
+
                     my %service_info = %{ $self->normalize_config( $arg ) };
                     push @out, $self->create_service( '$anonymous', %service_info );
                 }
@@ -357,7 +386,7 @@ sub find_refs {
 
 
 sub is_meta {
-    my ( $self, $arg ) = @_;
+    my ( $self, $arg, $root ) = @_;
 
     # Only a hashref can be meta
     return unless ref $arg eq 'HASH';
@@ -370,7 +399,7 @@ sub is_meta {
 
     # A regular service does not need the prefix, but must consist
     # only of meta keys
-    return 1 if scalar @keys eq grep { $meta{ $_ } } @keys;
+    return 1 if $root && scalar @keys eq grep { $meta{ $_ } } @keys;
 
     # A meta service contains at least one of these keys, as these are
     # the keys that can create a service. All other keys are
@@ -599,7 +628,7 @@ sub validate {
 
 
 package Beam::Wire::Exception;
-$Beam::Wire::Exception::VERSION = '1.017';
+$Beam::Wire::Exception::VERSION = '1.018';
 use Moo;
 with 'Throwable';
 use Types::Standard qw( :all );
@@ -612,7 +641,7 @@ has error => (
 
 
 package Beam::Wire::Exception::Constructor;
-$Beam::Wire::Exception::Constructor::VERSION = '1.017';
+$Beam::Wire::Exception::Constructor::VERSION = '1.018';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -625,7 +654,7 @@ has attr => (
 
 
 package Beam::Wire::Exception::Config;
-$Beam::Wire::Exception::Config::VERSION = '1.017';
+$Beam::Wire::Exception::Config::VERSION = '1.018';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -653,7 +682,7 @@ has '+error' => (
 
 
 package Beam::Wire::Exception::Service;
-$Beam::Wire::Exception::Service::VERSION = '1.017';
+$Beam::Wire::Exception::Service::VERSION = '1.018';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -671,7 +700,7 @@ has file => (
 
 
 package Beam::Wire::Exception::NotFound;
-$Beam::Wire::Exception::NotFound::VERSION = '1.017';
+$Beam::Wire::Exception::NotFound::VERSION = '1.018';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 
@@ -687,7 +716,7 @@ has '+error' => (
 
 
 package Beam::Wire::Exception::InvalidConfig;
-$Beam::Wire::Exception::InvalidConfig::VERSION = '1.017';
+$Beam::Wire::Exception::InvalidConfig::VERSION = '1.018';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 use overload q{""} => sub {
@@ -713,7 +742,7 @@ Beam::Wire - Lightweight Dependency Injection Container
 
 =head1 VERSION
 
-version 1.017
+version 1.018
 
 =head1 SYNOPSIS
 
@@ -1026,17 +1055,20 @@ dependencies are created first.
 
 =head2 is_meta
 
-    my $is_meta = $wire->is_meta( $ref_hash );
+    my $is_meta = $wire->is_meta( $ref_hash, $root );
 
 Returns true if the given hash reference describes some kind of
 Beam::Wire service. This is used to identify service configuration
 hashes inside of larger data structures.
 
 A service hash reference must contain at least one key, and must either
-be made completely of meta keys (as returned by L<the get_meta_names
-method|/get_meta_names>), or contain a L<prefixed|/meta_prefix> key that
-could create or reference an object (one of C<class>, C<extends>,
-C<config>, C<value>, or C<ref>);
+contain a L<prefixed|/meta_prefix> key that could create or reference an
+object (one of C<class>, C<extends>, C<config>, C<value>, or C<ref>) or,
+if the C<$root> flag exists, be made completely of unprefixed meta keys
+(as returned by L<the get_meta_names method|/get_meta_names>).
+
+The C<$root> flag is used by L<the get method|/get> to allow unprefixed
+meta keys in the top-level hash values.
 
 =head2 get_meta_names
 
@@ -1159,6 +1191,16 @@ Both "value" and "class" or "extends" are defined. These are mutually-exclusive.
 
 =back
 
+=head1 ENVIRONMENT VARIABLES
+
+=over 4
+
+=item BEAM_WIRE_DEBUG
+
+If set, print a bunch of internal debugging information to STDERR.
+
+=back
+
 =head1 AUTHORS
 
 =over 4
